# -*- coding: utf-8 -*-

"""Calculators classes"""

import wx
import wx.xrc as xrc


class IdealBody:
    """Ideal Body Measurements Calculator"""
    def __init__(self, parent):
        self.resource = xrc.XmlResource('xrc/calculators.xrc')
        self.dlg = self.resource.LoadDialog(parent, 'Calc01Dialog')

        self.wristBox = xrc.XRCCTRL(self.dlg, 'wristBox')
        self.wristChoice = xrc.XRCCTRL(self.dlg, 'wristChoice')
        self.chestBox = xrc.XRCCTRL(self.dlg, 'chestBox')
        self.waistBox = xrc.XRCCTRL(self.dlg, 'waistBox')
        self.hipsBox = xrc.XRCCTRL(self.dlg, 'hipsBox')
        self.armsBox = xrc.XRCCTRL(self.dlg, 'armsBox')
        self.forearmsBox = xrc.XRCCTRL(self.dlg, 'forearmsBox')
        self.thighBox = xrc.XRCCTRL(self.dlg, 'thighBox')
        self.calveBox = xrc.XRCCTRL(self.dlg, 'calveBox')
        self.neckBox = xrc.XRCCTRL(self.dlg, 'neckBox')
        self.dlg.FindWindowByName('wxID_CLOSE').SetId(wx.ID_CLOSE)

        self.prevSel = 0
        
        self.dlg.Bind(wx.EVT_TEXT, self.OnCalc, self.wristBox)
        self.dlg.Bind(wx.EVT_CHOICE, self.OnUnitChange, self.wristChoice)
        self.dlg.Bind(wx.EVT_BUTTON, self.OnClose, id=wx.ID_CLOSE)

    def OnCalc(self, event):
        array = []
        try:
            array.append(float(self.wristBox.GetValue()))
        except ValueError:
            array.append(0)

        if array[0] != 0:
            array.append(array[0] * 6.5)
            array.append(array[1] * 0.85)
            array.append(array[1] * 0.70)
            array.append(array[1] * 0.53)
            array.append(array[1] * 0.37)
            array.append(array[1] * 0.36)
            array.append(array[1] * 0.34)
            array.append(array[1] * 0.29)

            for i in xrange(0, 9):
                array[i] = round((array[i] + 0.0001) * 100, 1)
                array[i] = round(array[i] / 100.0, 1)

            self.chestBox.SetValue(str(array[1]))
            self.hipsBox.SetValue(str(array[2]))
            self.waistBox.SetValue(str(array[3]))
            self.thighBox.SetValue(str(array[4]))
            self.neckBox.SetValue(str(array[5]))
            self.armsBox.SetValue(str(array[6]))
            self.calveBox.SetValue(str(array[7]))
            self.forearmsBox.SetValue(str(array[8]))
        else:
            self.wristBox.SetValue('')
            self.chestBox.SetValue('')
            self.hipsBox.SetValue('')
            self.waistBox.SetValue('')
            self.thighBox.SetValue('')
            self.neckBox.SetValue('')
            self.armsBox.SetValue('')
            self.calveBox.SetValue('')
            self.forearmsBox.SetValue('')

    def OnUnitChange(self, event):
        currSel = self.wristChoice.GetSelection()
        if currSel == 0 and self.prevSel != 0:
            self.prevSel = currSel
            try:
                value = float(self.wristBox.GetValue())
            except ValueError:
                return
            value = round(value * 2.54, 1)
            self.wristBox.SetValue(str(value))
        elif currSel == 1 and self.prevSel != 1:
            self.prevSel = currSel
            try:
                value = float(self.wristBox.GetValue())
            except ValueError:
                return
            value = round(value / 2.54, 1)
            self.wristBox.SetValue(str(value))
            
    def OnClose(self, event):
        self.dlg.Destroy()


class BMICalc:
    """Body Mass Index calculator for average people only"""
    def __init__(self, parent):
        self.resource = xrc.XmlResource('xrc/calculators.xrc')
        self.dlg = self.resource.LoadDialog(parent, 'CalcBMIDialog')

        self.maleRadio = xrc.XRCCTRL(self.dlg, 'maleRadio')
        self.femaleRadio = xrc.XRCCTRL(self.dlg, 'femaleRadio')
        self.weightBox = xrc.XRCCTRL(self.dlg, 'weightBox')
        self.weightChoice = xrc.XRCCTRL(self.dlg, 'weightChoice')
        self.heightBox = xrc.XRCCTRL(self.dlg, 'heightBox')
        self.heightChoice = xrc.XRCCTRL(self.dlg, 'heightChoice')
        self.bmiBox = xrc.XRCCTRL(self.dlg, 'bmiBox')
        self.descBox = xrc.XRCCTRL(self.dlg, 'descBox')
        self.dlg.FindWindowByName('wxID_CLOSE').SetId(wx.ID_CLOSE)

        self.weightPrevSel = 0
        self.heightPrevSel = 0

        self.dlg.Bind(wx.EVT_RADIOBUTTON, self.OnRadioBtn, self.maleRadio)
        self.dlg.Bind(wx.EVT_RADIOBUTTON, self.OnRadioBtn, self.femaleRadio)
        self.dlg.Bind(wx.EVT_TEXT, self.OnCalc, self.weightBox)
        self.dlg.Bind(wx.EVT_TEXT, self.OnCalc, self.heightBox)
        self.dlg.Bind(wx.EVT_CHOICE, self.OnUnitHeightChange,
                      self.heightChoice)
        self.dlg.Bind(wx.EVT_CHOICE, self.OnUnitWeightChange,
                      self.weightChoice)
        self.dlg.Bind(wx.EVT_BUTTON, self.OnClose, id=wx.ID_CLOSE)

    def OnCalc(self, event):
        try:
            weight = (float(self.weightBox.GetValue()))
        except ValueError:
            weight = 0
        try:
            height = (float(self.heightBox.GetValue()))
        except ValueError:
            height = 0

        if weight != 0 and height != 0:
            if self.weightChoice.GetSelection() == 1:
                weight = weight * 0.45359
            if self.heightChoice.GetSelection() == 1:
                height = height * 2.54

            bmi = weight / ((height / 100)**2)

            desc = self.BodyDesc(bmi)
                           
            self.bmiBox.SetValue(str(round(bmi,1)))
            self.descBox.SetValue(desc)
        else:
            if weight == 0:
                self.weightBox.SetValue('')
            if height == 0:
                self.heightBox.SetValue('')
            self.bmiBox.SetValue('')
            self.descBox.SetValue('')

    def BodyDesc(self, bmi):
        if self.maleRadio.GetValue() == True:
            if bmi <= 20.7:
                desc = _("Underweight")
            elif bmi > 20.7 and bmi <= 26.4:
                desc = _("Normal")
            elif bmi > 26.4 and bmi <= 27.8:
                desc = _("Marginally overweight")
            elif bmi > 27.8 and bmi <= 31.1:
                desc = _("Overweight")
            elif bmi > 31.1 and bmi <= 45.4:
                desc = _("Severe overweight") 
            elif bmi > 45.4:
                desc = _("Morbid obesity")
        elif self.femaleRadio.GetValue() == True:
            if bmi <= 19.1:
                desc = _("Underweight")
            elif bmi > 19.1 and bmi <= 25.8:
                desc = _("Normal")
            elif bmi > 25.8 and bmi <= 27.3:
                desc = _("Marginally overweight")
            elif bmi > 27.3 and bmi <= 32.2:
                desc = _("Overweight")
            elif bmi > 31.1 and bmi <= 44.8:
                desc = _("Severe overweight") 
            elif bmi > 44.8:
                desc = _("Morbid obesity")
        return desc

    def OnUnitWeightChange(self, event):
        currSel = self.weightChoice.GetSelection()
        if currSel == 0 and self.weightPrevSel != 0:
            self.weightPrevSel = currSel
            try:
                value = float(self.weightBox.GetValue())
            except ValueError:
                return
            value = round(value * 0.45359, 1)
            self.weightBox.SetValue(str(value))
        elif currSel == 1 and self.weightPrevSel != 1:
            self.weightPrevSel = currSel
            try:
                value = float(self.weightBox.GetValue())
            except ValueError:
                return
            value = round(value / 0.45359, 1)
            self.weightBox.SetValue(str(value))

    def OnUnitHeightChange(self, event):
        currSel = self.heightChoice.GetSelection()
        if currSel == 0 and self.heightPrevSel != 0:
            self.heightPrevSel = currSel
            try:
                value = float(self.heightBox.GetValue())
            except ValueError:
                return
            value = round(value * 2.54, 1)
            self.heightBox.SetValue(str(value))
        elif currSel == 1 and self.heightPrevSel != 1:
            self.heightPrevSel = currSel
            try:
                value = float(self.heightBox.GetValue())
            except ValueError:
                return
            value = round(value / 2.54, 1)
            self.heightBox.SetValue(str(value))

    def OnRadioBtn(self, event):
        try:
            bmi = (float(self.bmiBox.GetValue()))
        except ValueError:
            bmi = 0

        if bmi != 0:
            desc = self.BodyDesc(bmi)
            self.descBox.SetValue(desc)

    def OnClose(self, event):
        self.dlg.Destroy()


class Cal2KJ:
    """Calorie KiloJoule Conversion"""
    def __init__(self, parent):
        self.resource = xrc.XmlResource('xrc/calculators.xrc')
        self.dlg = self.resource.LoadDialog(parent, 'CalcCal2KJDialog')

        self.caloriesBox = xrc.XRCCTRL(self.dlg, 'caloriesBox')
        self.kilojoulesBox = xrc.XRCCTRL(self.dlg, 'kilojoulesBox')
        self.dlg.FindWindowByName('wxID_CLOSE').SetId(wx.ID_CLOSE)

        self.prevEvent = 0

        self.dlg.Bind(wx.EVT_TEXT, self.OnCalc, self.caloriesBox)
        self.dlg.Bind(wx.EVT_TEXT, self.OnCalc, self.kilojoulesBox)
        self.dlg.Bind(wx.EVT_BUTTON, self.OnClose, id=wx.ID_CLOSE)

    def OnCalc(self, event):
        try:
            calories = (float(self.caloriesBox.GetValue()))
        except ValueError:
            calories = 0
        try:
            kilojoules = (float(self.kilojoulesBox.GetValue()))
        except ValueError:
            kilojoules = 0

        if event.GetId() == self.prevEvent or self.prevEvent == 0:
            if calories != 0 and event.GetId() == xrc.XRCID('caloriesBox'):
                kilojoules = round(calories * 4.184, 1)
                self.prevEvent = event.GetId()
                self.kilojoulesBox.SetValue(str(kilojoules))
            elif kilojoules != 0 and \
                     event.GetId() == xrc.XRCID('kilojoulesBox'):
                calories = round(kilojoules / 4.184, 1)
                self.prevEvent = event.GetId()
                self.caloriesBox.SetValue(str(calories)) 
            else:
                self.caloriesBox.SetValue('')
                self.kilojoulesBox.SetValue('')
            self.prevEvent = 0

    def OnClose(self, event):
        self.dlg.Destroy()
