# -*- coding: utf-8 -*-

"""Exercises tab for main frame's notebook"""

import wx
import wx.html
from wx import xrc
import wx.animate
import webbrowser
import  wx.lib.mixins.listctrl  as  listmix
import os, sys
from Hercules.exercises import *
from Hercules.gui.exercisedialog import *
from Hercules.exercises import Exercises
from Hercules.globals import imagesDir


class ExercisesTab(Exercises):
    """Exercises tab class"""

    def __init__(self, parent, resource):
        self.parent = parent
        self.resource = resource
        Exercises.__init__(self)
        
        self.utilityList = [_("Basic"), _("Auxiliary"),
                            _("Basic or Auxiliary")]
        self.mechanicsList = [_("Compound"), _("Isolated")]
        self.forceList = [_("Push"), _("Pull")]

        self.data = []

        self.InitList()
        self.InitButtons()
        self.InitOther()

    def InitButtons(self):
        """Init buttons on the exercises tab"""
        # Init buttons
        self.addNewExercise = xrc.XRCCTRL(self.parent, 'addNewExercise')
        self.editExercise = xrc.XRCCTRL(self.parent, 'editExercise')
        self.deleteExercise = xrc.XRCCTRL(self.parent, 'deleteExercise')

        # Set event handlers
        self.parent.Bind(wx.EVT_BUTTON, self.OnAddNewExercise,
                         self.addNewExercise)
        self.parent.Bind(wx.EVT_BUTTON, self.OnEditExercise,
                         self.editExercise)
        self.parent.Bind(wx.EVT_BUTTON, self.OnDeleteExercise,
                         self.deleteExercise)

        # goto buttons
        self.gotoFirst = xrc.XRCCTRL(self.parent, 'gotoFirst')
        self.gotoBack = xrc.XRCCTRL(self.parent, 'gotoBack')
        self.gotoForward = xrc.XRCCTRL(self.parent, 'gotoForward')
        self.gotoLast = xrc.XRCCTRL(self.parent, 'gotoLast')

        self.parent.Bind(wx.EVT_BUTTON, self.OnGotoForward,
                         self.gotoForward)
        self.parent.Bind(wx.EVT_BUTTON, self.OnGotoBack,
                         self.gotoBack)
        self.parent.Bind(wx.EVT_BUTTON, self.OnGotoFirst,
                         self.gotoFirst)
        self.parent.Bind(wx.EVT_BUTTON, self.OnGotoLast,
                         self.gotoLast)

    def InitList(self):
        """Init the exercises list"""
        # init list
        self.list = ListCtrlAutoWidth(self.parent, style=wx.LC_REPORT 
                                      | wx.BORDER_SUNKEN
                                      | wx.LC_SINGLE_SEL)

        self.resource.AttachUnknownControl("exercisesList", self.list,
                                           self.parent)

        # set up columns
        self.list.InsertColumn(0, _("Title"))
        self.list.InsertColumn(1, _("Utility"))
 
        # set up columns width
        self.list.SetColumnWidth(0, 250)
        self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)

        # load exercises from database into the list
        exercises = self.SelectExercises()
        if list(exercises) != []:
            for p in exercises:
                self.data.append([p.exerTitle, p.exerUtility, p.exerMechanics,
                                  p.exerForce, p.exerInstruct, p.exerMuscles,
                                  p.exerComments, p.exerImageFile, p.id])
            for data in self.data:
                self.InsertItem(data)

        # bind events
        self.parent.Bind(wx.EVT_LIST_ITEM_SELECTED,
                         self.OnItemSelected, self.list)

    def InitOther(self):
        """Init all other widgets"""
        # init text labels
        self.exerTitle = xrc.XRCCTRL(self.parent, 'exerciseTitle')
        self.exerUtility = xrc.XRCCTRL(self.parent, 'exerciseUtility')
        self.exerMechanics = xrc.XRCCTRL(self.parent, 'exerciseMechanics')
        self.exerForce = xrc.XRCCTRL(self.parent, 'exerciseForce')

        # init info windows
        self.exerInstruct = HtmlWindow(self.parent)
        self.exerMuscles = HtmlWindow(self.parent)
        self.exerComments = HtmlWindow(self.parent)

        self.resource.AttachUnknownControl("instructionHTML",
                                           self.exerInstruct,
                                           self.parent)
        self.resource.AttachUnknownControl("musclesHTML",
                                           self.exerMuscles,
                                           self.parent)
        self.resource.AttachUnknownControl("commentsHTML",
                                           self.exerComments,
                                           self.parent)
        
        self.exerImage = xrc.XRCCTRL(self.parent, 'exerciseImage')

        if len(self.data) >= 2:
            self.gotoFirst.Enable()
            self.gotoBack.Enable()
            self.gotoForward.Enable()
            self.gotoLast.Enable()
            self.index = -1
            
    def OnItemSelected(self, event):
        """Show the info about exercise when the iten selected"""
        self.index = event.m_itemIndex

        # get and set infos
        self.exerTitle.SetLabel(self.data[self.index][0])
        self.exerUtility.SetLabel(self.utilityList[self.data[self.index][1]])
        self.exerMechanics.SetLabel(self.mechanicsList[self.data[self.index][2]])
        self.exerForce.SetLabel(self.forceList[self.data[self.index][3]])
        # get and set more detailed infos
        self.exerInstruct.SetPage(self.data[self.index][4])
        self.exerMuscles.SetPage(self.data[self.index][5])
        self.exerComments.SetPage(self.data[self.index][6])
        # set image
        # clear image panel
        self.SetImage('')
        self.SetImage(os.path.join(imagesDir, self.data[self.index][7]))

        # activate delete and edit buttons
        self.editExercise.Enable()
        self.deleteExercise.Enable()

        # active or disactivate the first and back button
        if self.index == 0:
            self.gotoFirst.Disable()
            self.gotoBack.Disable()
        else:
            self.gotoFirst.Enable()
            self.gotoBack.Enable()
        # activate or disactivate the last button
        if self.index == len(self.data) - 1:
            self.gotoLast.Disable()
            self.gotoForward.Disable()
        else:
            self.gotoLast.Enable()
            self.gotoForward.Enable()

    def OnAddNewExercise(self, event):
        """Add new exercise event handler"""
        # Load new exercise dialog box
        dlg = AddExerciseDlg(self.parent, self.resource)
        retVal = dlg.ShowModal()
        if retVal == wx.ID_OK:
            exerciseInfo = dlg.GetExerciseInfo()
            if exerciseInfo != []:
                self.data.append(exerciseInfo)
                index = self.InsertItem(exerciseInfo)
                # insert new exercise into database
                self.InsertExerciseSQL(index)

    def OnEditExercise(self, event):
        """Edit selected exercise"""
        dlg = EditExerciseDlg(self.parent, self.resource,
                              self.data[self.index])
        retVal = dlg.ShowModal()
        if retVal == wx.ID_OK:
            exerciseInfo = dlg.GetExerciseInfo()
            # store old id
            exerciseInfo.append(self.data[self.index][8])
            self.data[self.index] = exerciseInfo
            self.UpdateItem(exerciseInfo, self.index)
            self.UpdateExerciseSQL(self.index)
            # update the view
            # get and set infos
            self.exerTitle.SetLabel(self.data[self.index][0])
            self.exerUtility.SetLabel(self.utilityList[self.data[self.index][1]])
            self.exerMechanics.SetLabel(self.mechanicsList[self.data[self.index][2]])
            self.exerForce.SetLabel(self.forceList[self.data[self.index][3]])
            # get and set more detailed infos
            self.exerInstruct.SetPage(self.data[self.index][4])
            self.exerMuscles.SetPage(self.data[self.index][5])
            self.exerComments.SetPage(self.data[self.index][6])
            # set image
            self.SetImage(os.path.join(imagesDir, self.data[self.index][7]))

    def OnDeleteExercise(self, event):
        """Delete selected exercise"""
        # Are you sure you want delete exercise
        msgStr = _("Are you sure you want delete exercise '%s'") % \
                  self.data[self.index][0]
        dlg = wx.MessageDialog(self.parent, msgStr,
                    _("Warning"), wx.OK | wx.CANCEL | wx.ICON_WARNING)
        retVal = dlg.ShowModal()
        if retVal == wx.ID_OK:
            # delete item from the list
            self.list.DeleteItem(self.index)
            # delete item from the database and self.data
            self.DeleteExerciseSQL(self.index)
            del self.data[self.index]

            # clear all labels and windows
            # deactivate delete and edit buttons
            self.editExercise.Disable()
            self.deleteExercise.Disable()
            # clear all info
            self.exerTitle.SetLabel('')
            self.exerUtility.SetLabel('')
            self.exerMechanics.SetLabel('')
            self.exerForce.SetLabel('')
            # get and set more detailed infos
            self.exerInstruct.SetPage('')
            self.exerMuscles.SetPage('')
            self.exerComments.SetPage('')
            # set null image
            self.SetImage('')

            dlg.Destroy()

    def OnGotoForward(self, event):
        """Goto forward in the exercise list"""
        if self.index < len(self.data) - 1:
            self.index += 1
            self.list.SetItemState(self.index, wx.LIST_STATE_SELECTED,
                                   wx.LIST_STATE_SELECTED)

    def OnGotoBack(self, event):
        """Goto backward in the exercise list"""
        if self.index > 0:
            self.index -= 1
            self.list.SetItemState(self.index, wx.LIST_STATE_SELECTED,
                                   wx.LIST_STATE_SELECTED)

    def OnGotoFirst(self, event):
        """Goto backward in the exercise list"""
        self.index = 0
        self.list.SetItemState(self.index, wx.LIST_STATE_SELECTED,
                               wx.LIST_STATE_SELECTED)

    def OnGotoLast(self, event):
        """Goto backward in the exercise list"""
        self.index = len(self.data) - 1
        self.list.SetItemState(self.index, wx.LIST_STATE_SELECTED,
                               wx.LIST_STATE_SELECTED)

    def InsertItem(self, exerciseInfo):
        """Insert new exercise into the list"""
        index = self.list.InsertStringItem(sys.maxint,
                                           exerciseInfo[0])
        self.list.SetStringItem(index, 1,
                                self.utilityList[exerciseInfo[1]])
        return index

    def UpdateItem(self, exerciseInfo, index):
        """Update selected exercise in the list"""
        self.list.SetStringItem(index, 0, exerciseInfo[0])
        self.list.SetStringItem(index, 1, self.utilityList[exerciseInfo[1]])

    def InsertExerciseSQL(self, index):
        """Insert exercise into database"""
        # insert new exercise
        exercise = self.InsertExercise(self.data[index][0],
                                       self.data[index][1],
                                       self.data[index][2],
                                       self.data[index][3],
                                       self.data[index][4],
                                       self.data[index][5],
                                       self.data[index][6],
                                       self.data[index][7])
        # store exercise id
        self.data[index].append(exercise.id)

    def UpdateExerciseSQL(self, index):
        """Update selected exercise in the database"""
        # update existed exercise
        exercise = self.GetExercise(self.data[index][8])
        self.UpdateExercise(exercise, self.data[index][0],
                            self.data[index][1],
                            self.data[index][2],
                            self.data[index][3],
                            self.data[index][4],
                            self.data[index][5],
                            self.data[index][6],
                            self.data[index][7])

    def DeleteExerciseSQL(self, index):
        """Delete exercise from the database"""
        # get exercise instance
        exercise = self.GetExercise(self.data[index][8])
        # delete exercise instance from database
        self.DeleteExercise(exercise)

    def SetImage(self, filename):
        sizer = wx.BoxSizer(wx.VERTICAL)
        aniCtrl = GIFAnimationCtrl(self.exerImage, size=(160,120),
                                   filename=filename)
        aniCtrl.GetPlayer().UseBackgroundColour(True)
        aniCtrl.Play()
        sizer.Add(aniCtrl, 1, wx.EXPAND)
        self.exerImage.SetSizerAndFit(sizer)


class ListCtrlAutoWidth(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
    def __init__(self, parent, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):
        wx.ListCtrl.__init__(self, parent, pos=pos, size=size, style=style)
        listmix.ListCtrlAutoWidthMixin.__init__(self)


class HtmlWindow(wx.html.HtmlWindow):
    def __init__(self, parent):
        wx.html.HtmlWindow.__init__(self, parent,
            style=wx.NO_FULL_REPAINT_ON_RESIZE | wx.SUNKEN_BORDER)
        if "gtk2" in wx.PlatformInfo:
            self.SetStandardFonts()
        
    def OnLinkClicked(self, link):
        a = link.GetHref()
        if a.startswith('#'):
            self.base_OnLinkClicked(link)
        else:
            webbrowser.open(a)
