# -*- coding: utf-8 -*-

"""Exercise Wizard for ExRx.net Exercise Directory"""

import wx
import wx.xrc as xrc
import wx.wizard as wiz
from Hercules.plugins.net_exrx import *


class WizardPageStart:
    def __init__(self, parent, page):
        self.pg = xrc.XRCCTRL(parent, page)

    def SetNext(self, next):
        self.next = next

    def SetPrev(self, prev):
        self.prev = prev

    def GetNext(self):
        return self.next

    def GetPrev(self):
        return self.prev


class WizardPageOne:
    def __init__(self, parent, page):
        self.pg = xrc.XRCCTRL(parent, page)

        self.rb = xrc.XRCCTRL(self.pg, 'radioBox')

        self.pg.Bind(wx.EVT_RADIOBOX, self.OnRadio, self.rb)

    def GetSelection(self):
        return self.rb.GetSelection()

    def OnRadio(self, event):
        nextPage = self.GetNext()
        nextPage.SetPage(self.GetSelection())
        nextPage.SetActive()

    def SetNext(self, next):
        self.next = next

    def SetPrev(self, prev):
        self.prev = prev

    def GetNext(self):
        return self.next

    def GetPrev(self):
        return self.prev


class WizardPageTwo:
    def __init__(self, parent, page, resource):
        self.resource = resource
        self.parent = parent
        self.page = None
        self.pageList = ['NeckPanel', 'ShouldersPanel', 'UpperArmsPanel',
                         'ForearmsPanel', 'BackPanel', 'ChestPanel',
                         'WaistPanel', 'HipsPanel', 'ThighsPanel',
                         'CalvesPanel']
        self.pg = xrc.XRCCTRL(parent, page)

        self.SetPage(0)

    def GetSelection(self):
        return self.rb.GetSelection()

    def SetPage(self, selection):
        self.pg.DestroyChildren()
        self.page = self.resource.LoadPanel(self.pg, self.pageList[selection])
        self.muscles = selection
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.page, 2, wx.EXPAND)
        self.pg.SetSizer(sizer)
        self.rb = xrc.XRCCTRL(self.page, 'radioBox')
        self.pg.Bind(wx.EVT_RADIOBOX, self.OnRadio, self.rb)
        
    def OnRadio(self, event):
        self.SetActive()

    def SetActive(self):
        nextPage = self.GetNext()
        nextPage.SetRadioBox(self.muscles, self.GetSelection())
        
    def SetNext(self, next):
        self.next = next

    def SetPrev(self, prev):
        self.prev = prev

    def GetNext(self):
        return self.next

    def GetPrev(self):
        return self.prev


class WizardPageThree:
    def __init__(self, parent, page):
        self.pg = xrc.XRCCTRL(parent, page)

        self.rb = xrc.XRCCTRL(self.pg, 'radioBox')

        self.SetRadioBox(0, 0)

    def GetSelection(self):
        return self.rb.GetSelection()

    def SetRadioBox(self, muscles, selection):
        firstActive = -1
        for i in xrange(0,8):
            if firstActive == -1 and exrxList[muscles][selection][i] == 1:
                firstActive = i
            self.rb.EnableItem(i, exrxList[muscles][selection][i])
        self.rb.SetSelection(firstActive)

    def SetNext(self, next):
        self.next = next

    def SetPrev(self, prev):
        self.prev = prev

    def GetNext(self):
        return self.next

    def GetPrev(self):
        return self.prev


class WizardPageFour:
    def __init__(self, parent, page):
        self.pg = xrc.XRCCTRL(parent, page)

        self.list = xrc.XRCCTRL(parent, 'exercisesList')

    def SetList(self, exercisesList):
        """Set up exercises list"""
        self.list.Set(exercisesList)
        self.list.SetSelection(0)

    def GetSelection(self):
        """Get selected exercise"""
        return self.list.GetSelection()

    def SetNext(self, next):
        self.next = next

    def SetPrev(self, prev):
        self.prev = prev

    def GetNext(self):
        return self.next

    def GetPrev(self):
        return self.prev


class WizardPageFinish:
    def __init__(self, parent, page):
        self.pg = xrc.XRCCTRL(parent, page)

    def SetNext(self, next):
        self.next = next

    def SetPrev(self, prev):
        self.prev = prev

    def GetNext(self):
        return self.next

    def GetPrev(self):
        return self.prev


class ExerciseWizard:
    """Exercise Wizard class"""
    
    def __init__(self, parent):
        self.parent = parent

        self.exercisesList = []
        self.exerciseInfo = []
        
        # Load resource file with wizard pages
        self.resource = xrc.XmlResource('xrc/exercisewizard.xrc')

        # Create the wizard and the pages
        self.wizard = self.resource.LoadObject(self.parent,
                                               'ExerciseWizard',
                                               'wxWizard')

        # Bind event handlers
        self.wizard.Bind(wiz.EVT_WIZARD_PAGE_CHANGING, self.OnWizPageChanging)
    
        self.pageS = WizardPageStart(self.wizard, 'StepStart')
        self.page1 = WizardPageOne(self.wizard, 'StepOne')
        self.page2 = WizardPageTwo(self.wizard, 'StepTwo', self.resource)
        self.page3 = WizardPageThree(self.wizard, 'StepThree')
        self.page4 = WizardPageFour(self.wizard, 'StepFour')
        self.pageF = WizardPageFinish(self.wizard, 'StepFinish')

        # Set the initial order of the pages
        self.pageS.SetNext(self.page1)
        self.page1.SetPrev(self.pageS)
        self.page1.SetNext(self.page2)
        self.page2.SetPrev(self.page1)
        self.page2.SetNext(self.page3)
        self.page3.SetPrev(self.page2)
        self.page3.SetNext(self.page4)
        self.page4.SetPrev(self.page3)
        self.page4.SetNext(self.pageF)
        self.pageF.SetPrev(self.page4)
    
        self.wizard.FitToPage(self.pageS.pg)
        if self.wizard.RunWizard(self.pageS.pg):
            self.isSuccess = True
        else:
            self.isSuccess = False

    def OnWizPageChanging(self, event):
        if event.GetDirection():
            direction = 'forward'
        else:
            direction = 'backward'

        page = event.GetPage()

        # fetch exercises list from online directory
        if page.GetId() == xrc.XRCID('StepThree') and direction == 'forward':
            self.exercisesList = LoadExercisesList(self.page2.GetSelection(),
                                                  self.page3.GetSelection(),
                                                  self.page1.GetSelection())
            if self.exercisesList == []:
                dlg = wx.MessageDialog(self.wizard,
                    _("Unable to fetch data!"
                      "\n\nPlease, check your Internet connection."),
                    _("Error"), wx.OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                event.Veto()
            else:
                tmpList = []
                # Extract exercises names
                for i in xrange(0, len(self.exercisesList)):
                    tmpList.append(self.exercisesList[i][0])
                self.page4.SetList(tmpList)
        # fetch all exercise info from the directory
        if page.GetId() == xrc.XRCID('StepFour') and direction == 'forward':
            item = self.page4.GetSelection()
            self.exerciseInfo = LoadExerciseInfo(self.exercisesList[item][1])
            if self.exerciseInfo == []:
                dlg = wx.MessageDialog(self.wizard,
                    _("Unable to fetch data!"
                      "\n\nPlease, check your Internet connection."),
                    _("Error"), wx.OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                event.Veto()

    def GetExerciseInfo(self):
        """Get exercise info"""
        if self.isSuccess == True:
            return self.exerciseInfo
        else:
            return []
