# -*- coding: utf-8 -*-

"""Main Frame class"""

import wx
from wx import xrc
from Hercules import hercules_version
from Hercules.gui.notebook import *
from Hercules.gui.calculators import *
from Hercules.globals import sharedir

class HerculesMainFrame:
    """Main frame class"""
    
    def __init__(self):
        # Create main frame
        self.resource = xrc.XmlResource(sharedir + 'xrc/hercules.xrc')

        self.InitFrame()
        self.InitMenubar()
        self.InitStatusbar()
        self.InitNotebook()

    def InitFrame(self):
        """Init main frame"""
        self.frame = self.resource.LoadFrame(None, 'MainFrame')
        self.frame.SetMinSize((880,660))

    def InitMenubar(self):
        """Init main menubar"""
        self.menuBar = self.resource.LoadMenuBar('MainMenuBar')
        # set menu event bindings
        self.frame.Bind(wx.EVT_MENU, self.OnQuit,
                        id=xrc.XRCID('itemQuit'))
        self.frame.Bind(wx.EVT_MENU, self.OnAbout,
                        id=xrc.XRCID('itemAbout'))
        self.frame.Bind(wx.EVT_MENU, self.OnCalc01,
                        id=xrc.XRCID('itemCalc01'))
        self.frame.Bind(wx.EVT_MENU, self.OnCalcBMI,
                        id=xrc.XRCID('itemCalcBMI'))
        self.frame.Bind(wx.EVT_MENU, self.OnCalcCal2KJ,
                        id=xrc.XRCID('itemCal2KJ'))
        self.frame.SetMenuBar(self.menuBar)

    def InitStatusbar(self):
        """Init status bar for main frame"""
        # Create a status bar
        self.frame.CreateStatusBar()
        # Set text for status bar
        self.frame.SetStatusText(_("Welcome to The Diary Of Hercules!"))

    def InitNotebook(self):
        """Init notebook's tabs"""
        self.notebook = HerculesNotebook(self.frame, self.resource)

    def OnAbout(self, event):
        """About dialog box"""
        dlg = wx.MessageDialog(self.frame, "The Diary of Hercules " + \
                    hercules_version + 
                    _("\n\nPersonal workout diary for bodybuilders"
                      "\n\nCopyright (c) Basil Shubin\n"
                      "Email: <bashu@users.sourceforge.net>\n"
                      "Homepage: http://hercules.sourceforge.net"),
                    _("About The Diary of Hercules"), wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()                                                        
        dlg.Destroy()

    def OnQuit(self, event):
        """Quits the program"""
        self.frame.Close()

    def OnCalc01(self, event):
        """Calculate your ideal body measurement"""
        dlg = IdealBody(self.frame)

        ret_val = dlg.dlg.ShowModal()
        if ret_val == wx.ID_EXIT:
            dlg.dlg.Destroy()

    def OnCalcBMI(self, event):
        """BMI calculator for average people only"""
        dlg = BMICalc(self.frame)

        ret_val = dlg.dlg.ShowModal()
        if ret_val == wx.ID_EXIT:
            dlg.dlg.Destroy()

    def OnCalcCal2KJ(self, event):
        """Calorie KiloJoule conversion"""
        dlg = Cal2KJ(self.frame)

        ret_val = dlg.dlg.ShowModal()
        if ret_val == wx.ID_EXIT:
            dlg.dlg.Destroy()
